@@ -1,7 +1,10 @@
 # This file is part of Tryton.  The COPYRIGHT file at the top level of
 # this repository contains the full copyright notices and license terms.
 from decimal import Decimal
+
+from trytond.i18n import gettext
 from trytond.model import Workflow, ModelView, fields, Check
+from trytond.model.exceptions import AccessError
 from trytond.pool import Pool, PoolMeta
 from trytond.transaction import Transaction
 
@@ -21,12 +24,8 @@ class Move(metaclass=PoolMeta):
         cls._sql_constraints += [
             ('check_fifo_quantity_out',
                 Check(t, t.quantity >= t.fifo_quantity),
-                'FIFO quantity can not be greater than quantity.'),
+                'product_cost_fifo.msg_move_fifo_quantity_greater'),
             ]
-        cls._error_messages.update({
-                'del_move_fifo': ('You can not delete move "%s" that is used '
-                    'for FIFO cost price.'),
-                })
 
     @staticmethod
     def default_fifo_quantity():
@@ -52,33 +51,25 @@ class Move(metaclass=PoolMeta):
         to_save = []
         for move, move_qty in fifo_moves:
             consumed_qty += move_qty
-            if move.from_location.type in {'supplier', 'production'}:
-                with Transaction().set_context(date=move.effective_date):
-                    move_unit_price = Currency.compute(
-                        move.currency, move.unit_price,
-                        self.company.currency, round=False)
-                move_unit_price = Uom.compute_price(
-                    move.uom, move_unit_price, move.product.default_uom)
-            else:
-                move_unit_price = move.cost_price or 0
+
+            with Transaction().set_context(date=move.effective_date):
+                move_unit_price = Currency.compute(
+                    move.currency, move.unit_price,
+                    self.company.currency, round=False)
+            move_unit_price = Uom.compute_price(move.uom, move_unit_price,
+                    move.product.default_uom)
             cost_price += move_unit_price * Decimal(str(move_qty))
 
             move_qty = Uom.compute_qty(self.product.default_uom, move_qty,
                     move.uom, round=False)
             move.fifo_quantity = (move.fifo_quantity or 0.0) + move_qty
-            # Due to float, the fifo quantity result can exceed the quantity.
-            assert move.quantity >= move.fifo_quantity - move.uom.rounding
-            move.fifo_quantity = min(move.fifo_quantity, move.quantity)
             to_save.append(move)
         if to_save:
             # TODO save in do method when product change
             self.__class__.save(to_save)
 
-        if consumed_qty:
+        if Decimal(str(consumed_qty)) != Decimal("0"):
             cost_price = cost_price / Decimal(str(consumed_qty))
-        else:
-            cost_price = self.product.get_multivalue(
-                'cost_price', **self._cost_price_pattern)
 
         # Compute average cost price
         unit_price = self.unit_price
@@ -87,7 +78,7 @@ class Move(metaclass=PoolMeta):
         average_cost_price = self._compute_product_cost_price('out')
         self.unit_price = unit_price
 
-        if cost_price:
+        if cost_price != Decimal("0"):
             digits = self.__class__.cost_price.digits
             cost_price = cost_price.quantize(
                 Decimal(str(10.0 ** -digits[1])))
@@ -127,14 +118,7 @@ class Move(metaclass=PoolMeta):
                 ('fifo_quantity', '!=', 0.0),
                 ])
         if fifo_moves:
-            cls.raise_user_error('del_move_fifo', (fifo_moves[0].rec_name,))
+            raise AccessError(
+                gettext('product_cost_fifo.msg_move_delete_fifo',
+                    move=fifo_moves[0].rec_name))
         super(Move, cls).delete(moves)
-
-    @classmethod
-    def copy(cls, moves, default=None):
-        if default is None:
-            default = {}
-        else:
-            default = default.copy()
-        default.setdefault('fifo_quantity', cls.default_fifo_quantity())
-        return super().copy(moves, default=default)
